package jp.oarts.pirka.core.ctrl.proc.def;

import java.util.List;
import java.util.Map;

import jp.oarts.pirka.core.analyzer.html.HtmlTools;
import jp.oarts.pirka.core.ctrl.proc.CtrlProcesser;
import jp.oarts.pirka.core.err.ErrorMessageManagerIF;
import jp.oarts.pirka.core.err.ErrorMessageStocker;
import jp.oarts.pirka.core.general.HtmlCtrlParts;
import jp.oarts.pirka.core.general.HtmlParts;
import jp.oarts.pirka.core.general.WindowBaseData;
import jp.oarts.pirka.core.util.field.FieldItem;

/**
 * SPAN^OvZbT[
 * 
 * @author ito
 * 
 */
public class SpanProcesser extends CtrlProcesser {

	/**
	 * HTMLp[cXg̎^O̎qp[c̃^O鎞trueԂ
	 * 
	 * @return true=qp[c̃^O
	 */
	@Override
	public boolean isUseChild() {
		return true;
	}

	/**
	 * [v^O̎tureԂ
	 * 
	 * @return true=[v^O
	 */
	@Override
	public boolean isLoop() {
		return false;
	}

	/**
	 * XCb`^O̎tureԂ
	 * 
	 * @return true=XCb`^O
	 */
	public boolean isSwitch() {
		return false;
	}

	/**
	 * uEUœ͂sȂ^ÔƂrtueԂ
	 * 
	 * @return true=uEUœ
	 */
	@Override
	public boolean isInputCtrl() {
		return false;
	}

	/**
	 * uEU[̃ANVƂtureԂ
	 * 
	 * @return true=ANV
	 */
	@Override
	public boolean isActionCtrl() {
		return false;
	}

	/**
	 * O[sOsȂ^OiWI{^Ȃǁj̊evf̎tureԂ
	 * 
	 * @return true=O[sOvf
	 */
	public boolean isGroupItem() {
		return false;
	}

	/**
	 * `FbNsȂRg[̎tureԂ
	 * 
	 * @return true=`FbNsȂRg[
	 */
	public boolean isCheckCtrl() {
		return false;
	}

	/**
	 * tH[}bg\ȃRg[̎tureԂ
	 * 
	 * @return true=tH[}bg\ȃRg[
	 */
	public boolean isFormatCtrl() {
		return true;
	}

	/**
	 * f[^̃ZbgsȂ
	 * 
	 * @param data
	 */
	public void setValue(Object data, FieldItem fieldItem) {
		if (data == null || !(data instanceof String)) {
			throw new RuntimeException("w肳ꂽl̓Zbgł܂ StringIuWFNgZbgĂ");
		}

		fieldItem.setValue((String) data);
	}

	/**
	 * l͂
	 * 
	 * @param ctrl
	 *            Rg[
	 * @param list
	 *            ̃^OHTMLp[c̃Xg
	 */
	@Override
	public void analyzeInitValue(HtmlCtrlParts ctrl, List<HtmlParts> list) {
		ctrl.setDefaultValue(HtmlTools.toSimpleString(list));
		ctrl.setTagEffective(true);
		ctrl.setWebSafe(true);
	}

	/**
	 * ^O쐬
	 * 
	 * @param ctrl
	 *            Rg[p[c
	 * @param loopSuffix
	 *            [vTtBbNX
	 * @param value
	 *            l
	 * @param checked
	 *            `FbNl
	 * @param windowBaseData
	 *            ʃf[^
	 * @param errorMessageManager
	 *            G[bZ[W}l[W[
	 * @param errorMessageStocker
	 *            G[bZ[WXgbJ[
	 * @param errorFlag
	 *            G[tO true=G[
	 * @return 쐬ꂽ^O
	 */
	public String makeStartTag(HtmlCtrlParts ctrl, String loopSuffix, String value, boolean checked, WindowBaseData windowBaseData, //
			ErrorMessageManagerIF errorMessageManager, ErrorMessageStocker errorMessageStocker, boolean errorFlag) {

		StringBuilder sb = new StringBuilder();

		if (ctrl.isTagEffective()) {
			Map<String, String> optionMap = ctrl.getOptionMap();
			if (errorFlag) {
				optionMap = changeStyle(errorMessageManager.getNgLabelStyle(), optionMap);
			}
			sb.append('<');
			sb.append(ctrl.getTagName());
			sb.append(createOptionString(ctrl.getTypeOption(), ctrl.getName() + loopSuffix, loopSuffix, null, ctrl.isSuffix(), optionMap));
			sb.append(">");
		}

		if (ctrl.getBaseName().equals(ErrorMessageManagerIF.ERROR_MESSAGE_FIELD_BASE_NAME)) {
			// G[bZ[W
			if (errorMessageManager.isWebSafeErrorMessage()) {
				sb.append(HtmlTools.webString(errorMessageManager.createErrorMessage(errorMessageStocker)));
			} else {
				sb.append(errorMessageManager.createErrorMessage(errorMessageStocker));
			}
		} else {
			if (ctrl.isWebSafe()) {
				sb.append(HtmlTools.webString(value));
			} else {
				sb.append(value);
			}
		}
		return sb.toString();
	}

}
